// SDL+OpenAL.cpp

#include "stdafx.h"

#pragma comment(lib, "SDL.lib")
#pragma comment(lib, "SDLmain.lib")
#pragma comment(lib, "OpenGL32.lib")
#pragma comment(lib, "glu32.lib")
#pragma comment(lib, "OpenAL32.lib")
#pragma comment(lib, "ALut.lib")
#ifdef _DEBUG
	#pragma comment(linker, "/nodefaultlib:msvcrt.lib")
#endif

const int SCREEN_WIDTH  = 640;
const int SCREEN_HEIGHT = 480;

namespace Global
{
	SDL_Surface*	screen = NULL;
	GLUquadricObj*	quadric = NULL;

	ALuint	sound;
	ALuint	buffer;

	bool	shouldRotate = true;
	float	angle;
	float	velocity;
	float	radius;
}

void terminate()
{
	alSourceStop(Global::sound);
	alDeleteSources(1, &Global::sound);
	alDeleteBuffers(1, &Global::buffer);
	gluDeleteQuadric(Global::quadric);
}

ALuint LoadSoundBuffer(const char* filename);
void ProcessEvents();
void Render();

int main(int argc, char** argv)
{
	// SDL
	if(SDL_Init(SDL_INIT_VIDEO) < 0)
		return -1;
	atexit(SDL_Quit);

	// SDL_OpenGL
	SDL_GL_SetAttribute(SDL_GL_RED_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_GREEN_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_BLUE_SIZE, 5);
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 16);
	SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
	Global::screen = SDL_SetVideoMode(SCREEN_WIDTH, SCREEN_HEIGHT, 0, SDL_OPENGL);
	if(Global::screen == NULL)
		return -1;

	// OpenGL
    glShadeModel(GL_SMOOTH);
    glCullFace(GL_BACK);
    glFrontFace(GL_CCW);
    glEnable(GL_CULL_FACE);
	glEnable(GL_DEPTH_TEST);
    glClearColor(0, 0, 0, 0);
    glViewport(0, 0, SCREEN_WIDTH, SCREEN_HEIGHT);
	Global::quadric = gluNewQuadric();
	// OpenGL : Projection
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluPerspective(60.0, (float)SCREEN_WIDTH/(float)SCREEN_HEIGHT, 1.0, 1024.0);
	// OpenGL : Lighting
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// OpenAL
	alutInit(NULL, 0);
	atexit(alutExit);
	// OpenAL : Xi
	alListenerf(AL_GAIN, 1.0);
	alDopplerFactor(1.0); // 1.0Ȃ狭ȂB傫ق킩₷
	alDopplerVelocity(343); // using meters/second
	// OpenAL : obt@
	Global::buffer = LoadSoundBuffer("motor_b8.wav");
	// OpenAL : \[X
	alGenSources(1, &Global::sound);
	alSourcei(Global::sound, AL_BUFFER, Global::buffer);
	alSourcei(Global::sound, AL_LOOPING, AL_TRUE);

	// init object
	atexit(terminate);
	Global::angle    = 0;
	Global::velocity = 1.0; // rad/sec
	Global::radius   = 3;
	alSourcePlay(Global::sound);

	// Event Loop
    while(true)
	{
        ProcessEvents();
        Render();
    }
	return 0;
}

ALuint LoadSoundBuffer(const char* filename)
{
	ALuint buffer;
	ALenum	format;
	ALvoid* data;
	ALsizei size, freq;
	ALboolean loop;
	// WAVE̓ǂݍ
	alutLoadWAVFile(const_cast<ALbyte*>(filename), &format, &data, &size, &freq, &loop);
	if(alGetError() != AL_NO_ERROR)
		return 0;
	// obt@̍쐬
	alGenBuffers(1, &buffer);
	alBufferData(buffer, format, data, size, freq);
	// f[^͂Ȃ
	alutUnloadWAV(format, data, size, freq);
	return buffer;
}

void ProcessEvents()
{
	SDL_Event e;
	while(SDL_PollEvent(&e))
	{
		switch(e.type)
		{
		case SDL_KEYDOWN:
			switch(e.key.keysym.sym)
			{
			case SDLK_ESCAPE:
				exit(0);
				break;
			case SDLK_SPACE:
				Global::shouldRotate = !Global::shouldRotate;
				break;
			}
			break;
		case SDL_QUIT:
			exit(0);
			break;
		}
	}
}

void UpdateObject(float interval)
{
	float v[4];

	// IuWFNg̈ʒu
	v[0] = Global::radius * cosf(Global::angle);
	v[1] = 0;
	v[2] = Global::radius * sinf(Global::angle);
	v[3] = 0;
	glTranslatef(v[0], v[1], v[2]);
	gluSphere(Global::quadric, 1.0, 8, 8);

	float pos[16];
	glGetFloatv(GL_MODELVIEW_MATRIX, pos);
	alSourcefv(Global::sound, AL_POSITION, pos+12);

	// IuWFNg̑x
	if(Global::shouldRotate)
	{
		Global::angle += Global::velocity * interval;
		v[0] = -Global::velocity * sinf(Global::angle);
		v[1] = 0;
		v[2] = Global::velocity * cosf(Global::angle);
	}
	else
	{
		v[0] = v[1] = v[2] = 0;
	}
	alSourcefv(Global::sound, AL_VELOCITY, v);
}

void Render()
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    glMatrixMode(GL_MODELVIEW);

	static Uint32 last = SDL_GetTicks();
	Uint32 now = SDL_GetTicks();
	float interval = (now - last) * 0.001f;
	last = now;

	// Cg}eA
	const GLfloat LIGHT_AMBIENT[4]  = {0.2f, 0.2f, 0.2f, 1.0f};
	const GLfloat LIGHT_POSITION[4] = {1.0f, 1.0f, 1.0f, 1.0f};
	const GLfloat LIGHT_DIFFUSE[4]  = {1.0f, 1.0f, 1.0f, 1.0f};
	const GLfloat OBJECT_DIFFUSE[4] = {1.0f, 0.5f, 0.5f, 1.0f};
	glLoadIdentity();
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, LIGHT_AMBIENT);
	glLightfv(GL_LIGHT0, GL_POSITION, LIGHT_POSITION);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, LIGHT_DIFFUSE);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, OBJECT_DIFFUSE);

	// OpenAL Listener
	ALfloat listenerPos[]={0,0,0};
	ALfloat listenerVel[]={0,0,0};
	ALfloat	listenerOri[]={0,0.0, -1,0.0, 1,0.0};
	alListenerfv(AL_POSITION, listenerPos);
	alListenerfv(AL_VELOCITY, listenerVel);
	alListenerfv(AL_ORIENTATION, listenerOri);

	// IuWFNg
	glLoadIdentity();
	glTranslatef(0.0, 0.0, -5.0);
	UpdateObject(interval);

    SDL_GL_SwapBuffers();
}
